rWeibull<-function(n,scale,shape,location,ltrunc,rtrunc,rev=F,minZero=T)
{
#
# this function generates random samples from a Weibull distribution
#    depending upon which parameters are supplied, the Weibull model switches
#
#  n = number of onservations to generate
#  scale = scale parameter
#  shape = shape parameter
#  location = location parameter (is missing, then two parameter family)
#  ltrunc = left truncation point, if missing then untruncated family
#  rtrunc = right truncation point, if missing then untruncated family)
#     if both ltrunc and rtrunc specified, then doubly truncated model
#  rev = logical flag for using reversed weibull (location = maximum required)
#
   if(missing(n)){stop("Error: number of observations not specified")}
   if(missing(scale)){stop("Error: scale parameter not specified")}
   if(missing(shape)){stop("Error: shape parameter not specified")}
   scale<-(1/scale)^shape
   if(!rev){
#
#  Normal Weibull Distribution
#
      if(missing(location)){location<-0}
      if(missing(ltrunc)){
         ltrunc<-0
      }else{
         if(ltrunc<location){stop("Error: left truncation must be greater than or equal to location parameter")}
         ltrunc<-ltrunc-location
      }
      Fl<-1-exp(-scale*ltrunc^shape)
      MinX<-location+ltrunc
      if(missing(rtrunc)){
         rtrunc<-Inf
      }else{
         if(rtrunc<location){stop("Error: right truncation must be greater than location parameter")}
         if(!missing(ltrunc)){
            if(rtrunc<ltrunc){stop("Error: right truncation must be greater than left truncation")}
         }
         rtrunc<-rtrunc-location
      }
      Fr<-1-exp(-scale*rtrunc^shape)
      MaxX<-location+rtrunc
   }else{
#
#  Reverse Weibull Distribution
#
      if(missing(location)){
         stop("Location must be specified for Reverse Weibull Distribution")
      }
      if(missing(ltrunc)){
         if(minZero){ltrunc<-location}else{ltrunc<-Inf}
      }else{
         if(ltrunc>location){stop("Error: left truncation must be less than Reverse location parameter")}
         ltrunc<-location-ltrunc
      }
      Fl<-exp(-scale*ltrunc^shape)
      MinX<-location-ltrunc
      if(missing(rtrunc)){
         rtrunc<-0
      }else{
         if(rtrunc>location){stop("Error: right truncation must be less than Reverse location parameter")}
         if(!missing(ltrunc)){
            if(rtrunc<location-ltrunc){stop("Error: right truncation must be greater than left truncation")}
         }
         rtrunc<-location-rtrunc
      }
      Fr<-exp(-scale*rtrunc^shape)
      MaxX<-location-rtrunc
   }
   Nunif<-runif(n)
   Nunif<-Fl+(Fr-Fl)*Nunif
   if(!rev){
      output<-location+((-log(1-Nunif)/scale)^(1/shape))
   }else{
#
# opps, reverse doesn't have the 1 -
#
#      output<-location-((-log(1-Nunif)/scale)^(1/shape))
      output<-location-((-log(Nunif)/scale)^(1/shape))
   }
   output[output<MinX]<-NA
   output[output>MaxX]<-NA
   while(any(is.na(output))){
      mm<-length(output[is.na(output)])
      Munif<-runif(mm)
      Munif<-Fl+(Fr-Fl)*Munif
      if(!rev){
         output[is.na(output)]<-location+((-log(1-Munif)/scale)^(1/shape))
      }else{
#         output[is.na(output)]<-location-((-log(1-Munif)/scale)^(1/shape))
         output[is.na(output)]<-location-((-log(Munif)/scale)^(1/shape))
      }
      output[output<MinX]<-NA
      output[output>MaxX]<-NA
   }
#####
   return(output)
}
pWeibull<-function(x,scale,shape,location,ltrunc,rtrunc,rev=F,minZero=T)
{
#
# this function generates cummulative probability densities from a Weibull distribution
#    depending upon which parameters are supplied, the Weibull model switches
#
#  x = Weibull variates
#  scale = scale parameter
#  shape = shape parameter
#  location = location parameter (is missing, then two parameter family)
#  ltrunc = left truncation point, if missing then untruncated family
#  rtrunc = right truncation point, if missing then untruncated family)
#     if both ltrunc and rtrunc specified, then doubly truncated model
#  rev = logical flag for using reversed weibull (location = maximum required)
#
   if(missing(x)){stop("Error: observations not specified")}
   if(missing(scale)){stop("Error: scale parameter not specified")}
   if(missing(shape)){stop("Error: shape parameter not specified")}
   if(missing(location)){location<-0}
   scale<-(1/scale)^shape
   if(!rev){
#
#  Normal Weibull Distribution
#
      if(missing(location)){location<-0}
      if(missing(ltrunc)){
         ltrunc<-0
      }else{
         if(ltrunc<location){stop("Error: left truncation must be greater than or equal to location parameter")}
         ltrunc<-ltrunc-location
      }
      Fl<-1-exp(-scale*ltrunc^shape)
      MinX<-location+ltrunc
      if(missing(rtrunc)){
         rtrunc<-Inf
      }else{
         if(rtrunc<location){stop("Error: right truncation must be greater than location parameter")}
         if(!missing(ltrunc)){
            if(rtrunc<ltrunc){stop("Error: right truncation must be greater than left truncation")}
         }
         rtrunc<-rtrunc-location
      }
      Fr<-1-exp(-scale*rtrunc^shape)
      MaxX<-location+rtrunc
   }else{
#
#  Reverse Weibull Distribution
#
      if(missing(location)){
         stop("Location must be specified for Reverse Weibull Distribution")
      }
      if(missing(ltrunc)){
         if(minZero){ltrunc<-location}else{ltrunc<-Inf}
      }else{
         if(ltrunc>location){stop("Error: left truncation must be less than Reverse location parameter")}
         ltrunc<-location-ltrunc
      }
      Fl<-exp(-scale*ltrunc^shape)
      MinX<-location-ltrunc
      if(missing(rtrunc)){
         rtrunc<-0
      }else{
         if(rtrunc>location){stop("Error: right truncation must be less than Reverse location parameter")}
         if(!missing(ltrunc)){
            if(rtrunc<location-ltrunc){stop("Error: right truncation must be greater than left truncation")}
         }
         rtrunc<-location-rtrunc
      }
      Fr<-exp(-scale*rtrunc^shape)
      MaxX<-location-rtrunc
   }
   if(!rev){
      xp<-x-location
      Fx<-1-exp(-scale*xp^shape)
   }else{
      xp<-location-x
      Fx<-exp(-scale*xp^shape)
   }
#   Fx<-1-exp(-scale*xp^shape)
   ppp<-(Fx-Fl)/(Fr-Fl)
   ppp[x<MinX]<-0
   ppp[x>MaxX]<-1
   return(ppp)
}
dWeibull<-function(x,scale,shape,location,ltrunc,rtrunc,rev=F,minZero=T)
{
#
# this function generates cummulative probability densities from a Weibull distribution
#    depending upon which parameters are supplied, the Weibull model switches
#
#  n = number of onservations to generate
#  scale = scale parameter
#  shape = shape parameter
#  location = location parameter (is missing, then two parameter family)
#  ltrunc = left truncation point, if missing then untruncated family
#  rtrunc = right truncation point, if missing then untruncated family)
#     if both ltrunc and rtrunc specified, then doubly truncated model
#  rev = logical flag for using reversed weibull (location = maximum required)
#
   if(missing(x)){stop("Error: observations not specified")}
   if(missing(scale)){stop("Error: scale parameter not specified")}
   if(missing(shape)){stop("Error: shape parameter not specified")}
   if(missing(location)){location<-0}
   scale<-(1/scale)^shape
   if(!rev){
#
#  Normal Weibull Distribution
#
      if(missing(location)){location<-0}
      if(missing(ltrunc)){
         ltrunc<-0
      }else{
         if(ltrunc<location){stop("Error: left truncation must be greater than or equal to location parameter")}
         ltrunc<-ltrunc-location
      }
      Fl<-1-exp(-scale*ltrunc^shape)
      MinX<-location+ltrunc
      if(missing(rtrunc)){
         rtrunc<-Inf
      }else{
         if(rtrunc<location){stop("Error: right truncation must be greater than location parameter")}
         if(!missing(ltrunc)){
            if(rtrunc<ltrunc){stop("Error: right truncation must be greater than left truncation")}
         }
         rtrunc<-rtrunc-location
      }
      Fr<-1-exp(-scale*rtrunc^shape)
      MaxX<-location+rtrunc
   }else{
#
#  Reverse Weibull Distribution
#
      if(missing(location)){
         stop("Location must be specified for Reverse Weibull Distribution")
      }
      if(missing(ltrunc)){
         if(minZero){ltrunc<-location}else{ltrunc<-Inf}
      }else{
         if(ltrunc>location){stop("Error: left truncation must be less than Reverse location parameter")}
         ltrunc<-location-ltrunc
      }
      Fl<-exp(-scale*ltrunc^shape)
      MinX<-location-ltrunc
      if(missing(rtrunc)){
         rtrunc<-0
      }else{
         if(rtrunc>location){stop("Error: right truncation must be less than Reverse location parameter")}
         if(!missing(ltrunc)){
            if(rtrunc<location-ltrunc){stop("Error: right truncation must be greater than left truncation")}
         }
         rtrunc<-location-rtrunc
      }
      Fr<-exp(-scale*rtrunc^shape)
      MaxX<-location-rtrunc
   }
   if(!rev){xp<-x-location}else{xp<-location-x}
   fx<-scale*shape*(xp^(shape-1))*exp(-scale*(xp^shape))
#   if(!rev){ppp<-fx/(Fr-Fl)}else{ppp<-fx/(Fl-Fr)}
   ppp<-fx/(Fr-Fl)
   ppp[x<MinX]<-0
   ppp[x>MaxX]<-0
   return(ppp)
}
qWeibull<-function(p,scale,shape,location,ltrunc,rtrunc,rev=F,minZero=T)
{
#
# this function generates random samples from a Weibull distribution
#    depending upon which parameters are supplied, the Weibull model switches
#
#  p = cummulative Weibull probabilities
#  scale = scale parameter
#  shape = shape parameter
#  location = location parameter (is missing, then two parameter family)
#  ltrunc = left truncation point, if missing then untruncated family
#  rtrunc = right truncation point, if missing then untruncated family)
#     if both ltrunc and rtrunc specified, then doubly truncated model
#  rev = logical flag for using reversed weibull (location = maximum required)
#
   if(missing(p)){stop("Error: probabilities not specified")}
   if(missing(scale)){stop("Error: scale parameter not specified")}
   if(missing(shape)){stop("Error: shape parameter not specified")}
   if(missing(location)){location<-0}
   scale<-(1/scale)^shape
   if(!rev){
#
#  Normal Weibull Distribution
#
      if(missing(location)){location<-0}
      if(missing(ltrunc)){
         ltrunc<-0
      }else{
         if(ltrunc<location){stop("Error: left truncation must be greater than or equal to location parameter")}
         ltrunc<-ltrunc-location
      }
      Fl<-1-exp(-scale*ltrunc^shape)
      MinX<-location+ltrunc
      if(missing(rtrunc)){
         rtrunc<-Inf
      }else{
         if(rtrunc<location){stop("Error: right truncation must be greater than location parameter")}
         if(!missing(ltrunc)){
            if(rtrunc<ltrunc){stop("Error: right truncation must be greater than left truncation")}
         }
         rtrunc<-rtrunc-location
      }
      Fr<-1-exp(-scale*rtrunc^shape)
      MaxX<-location+rtrunc
   }else{
#
#  Reverse Weibull Distribution
#
      if(missing(location)){
         stop("Location must be specified for Reverse Weibull Distribution")
      }
      if(missing(ltrunc)){
         if(minZero){ltrunc<-location}else{ltrunc<-Inf}
      }else{
         if(ltrunc>location){stop("Error: left truncation must be less than Reverse location parameter")}
         ltrunc<-location-ltrunc
      }
      Fl<-exp(-scale*ltrunc^shape)
      MinX<-location-ltrunc
      if(missing(rtrunc)){
         rtrunc<-0
      }else{
         if(rtrunc>location){stop("Error: right truncation must be less than Reverse location parameter")}
         if(!missing(ltrunc)){
            if(rtrunc<location-ltrunc){stop("Error: right truncation must be greater than left truncation")}
         }
         rtrunc<-location-rtrunc
      }
      Fr<-exp(-scale*rtrunc^shape)
      MaxX<-location-rtrunc
   }
   print(paste(MinX,MaxX))
   p<-Fl+(Fr-Fl)*p
   if(!rev){qqq<-location+((-log(1-p)/scale)^(1/shape))}else{qqq<-location-((-log(p)/scale)^(1/shape))}
   qqq[p<0]<-NA
   qqq[p>1]<-NA
   qqq[round(qqq,5)<MinX]<-NA
   qqq[round(qqq,5)>MaxX]<-NA
   return(qqq)
}
Weibull.parameter.finder<-function(data,wt)
{
   require(rpanel)
   do.reverse<-function(panel){
      if(panel$Irev){panel$location<-Xmax}else{panel$location<-0}
      hist.draw(panel)
      return(panel)
   }
   draw.fig1<-function(panel,probs){
      hist(panel$data,panel$Hbreaks,main="Histogram")
      Hcnnt<-hist(panel$data,panel$Hbreaks,plot=F)$counts
      nn<-length(probs)
      probs<-probs[2:nn]-probs[1:(nn-1)]
      xxx<-(panel$Hbreaks[2:nn]+panel$Hbreaks[1:(nn-1)])/2
      nn<-length(panel$data)
      lines(xxx,nn*probs,col="red",lwd=2)
   }
   draw.fig2<-function(panel){
      if(panel$Irev){mtit<-"Visual Reverse Weibull Parameters"}else{mtit<-"Visual Normal Weibull Parameters"}
      plot(0,0,type="n",axes=F,xlab="",ylab="",xlim=c(0,10),ylim=c(0,10),main=mtit)
      text(5,10,paste("Location=",round(panel$location,4)))
      text(5,8.5,paste("Scale=",round(panel$scale,4)))
      text(5,7,paste("Shape=",round(panel$shape,4)))
      if(panel$Iltrunc){
         text(5,5.5,paste("Left Truncation=",round(panel$ltrunc,4)))
      }
      if(panel$Irtrunc){
         text(5,4,paste("Right Truncation=",round(panel$rtrunc,4)))
      }
      text(5,2.5,paste("Shape Bounds: {",panel$ShapeMin,",",panel$ShapeMax,"}"))
      text(5,1,paste("Scale Bounds: {",panel$ScaleMin,",",panel$ScaleMax,"}"))
   }
   draw.fig3<-function(panel,probs){
      Hcnnt<-hist(panel$data,panel$Hbreaks,plot=F)$counts
      nn<-length(probs)
      probs<-probs[2:nn]-probs[1:(nn-1)]
      xxx<-(panel$Hbreaks[2:nn]+panel$Hbreaks[1:(nn-1)])/2
      nn<-length(panel$data)
      plot(xxx,Hcnnt-nn*probs,pch=16,xlab="X",ylab="Class Residuals")
      lines(xxx,rep(0,length(xxx)))
   }
   draw.fig4<-function(panel,probs){
      Hcnnt<-hist(panel$data,panel$Hbreaks,plot=F)$counts
      nn<-length(probs)
      probs<-probs[2:nn]-probs[1:(nn-1)]
      plot(0,0,type="n",axes=F,xlab="",ylab="",xlim=c(0,10),ylim=c(0,10),main="Visual Fit Statistics")
      mse<-sqrt(mean((Hcnnt-nn*probs)^2))
      text(5,10,paste("Root MSE=",round(mse,4)))
      if(panel$Iltrunc){
         if(panel$Irtrunc){
           ksout<-ks.test(panel$data[panel$data>=panel$ltrunc & panel$data<=panel$rtrunc],"pWeibull",scale=panel$scale,shape=panel$shape,location=panel$location,ltrunc=panel$ltrunc,rtrunc=panel$rtrunc,rev=panel$Irev)
         }else{
           ksout<-ks.test(panel$data[panel$data>=panel$ltrunc],"pWeibull",scale=panel$scale,shape=panel$shape,location=panel$location,ltrunc=panel$ltrunc,rev=panel$Irev)
         }
      }else{
         if(panel$Irtrunc){
           ksout<-ks.test(panel$data[panel$data<=panel$rtrunc],"pWeibull",scale=panel$scale,shape=panel$shape,location=panel$location,rtrunc=panel$rtrunc,rev=panel$Irev)
         }else{
           ksout<-ks.test(panel$data,"pWeibull",scale=panel$scale,shape=panel$shape,location=panel$location,rev=panel$Irev)
         }
      }
      text(5,8,paste("KS Test: D =",round(ksout$stat,4)))
      text(5,6,paste("Pr(D)=",round(ksout$p.val,4)))
   }
   hist.draw<-function(panel){
      par(mfrow=c(2,2))
      cwid<-panel$cwid
      Xmin<-0
      Xmax<-cwid*round(max(panel$data)/cwid+.5,0)
      panel$Hbreaks<-seq(Xmin,Xmax,cwid)
#      hist(panel$data,panel$Hbreaks,main="Histogram")
#      Hcnnt<-hist(panel$data,panel$Hbreaks,plot=F)$counts
#      if(panel$Irev){panel$location<-Xmax}else{panel$location<-0}
      if(panel$Iltrunc){
         if(!panel$Irev){
            if(panel$ltrunc<panel$location){panel$ltrunc<-panel$location}
         }else{
            if(panel$ltrunc>panel$location){panel$ltrunc<-panel$location}
         }
         if(panel$Irtrunc){
            if(panel$rtrunc<panel$ltrunc){panel$rtrunc<-panel$ltrunc}
            if(panel$Irev){if(panel$rtrunc>panel$location){panel$rtrunc<-panel$location}}
#
# doubly truncated
#
            probs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,rtrunc=panel$rtrunc,ltrunc=panel$ltrunc,rev=panel$Irev)
         }else{
#
# left truncated
#
            probs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,ltrunc=panel$ltrunc,rev=panel$Irev)
        
         }
      }else{
         if(panel$Irtrunc){
            if(!panel$Irev){
               if(panel$rtrunc<panel$location){panel$rtrunc<-panel$location}
            }else{
               if(panel$rtrunc>panel$location){panel$rtrunc<-panel$location}
            }
#
# right truncated
#
            probs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,rtrunc=panel$rtrunc,rev=panel$Irev)
         }else{
#
# full Weibull
#
            probs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,rev=panel$Irev)
         }
      }
      draw.fig1(panel,probs)
      draw.fig2(panel)
      draw.fig3(panel,probs)
      draw.fig4(panel,probs)
      return(panel)
   }
   get.MLEs<-function(panel)
   {
      if(panel$Iltrunc){
         if(panel$Irtrunc){
            MLEout<-Weibull.MLE(data=panel$data,Params=c(panel$scale,panel$shape),location=panel$location,ltrunc=panel$ltrunc,rtrunc=panel$rtrunc,rev=panel$Irev,Pmin=c(panel$ScaleMin,panel$ShapeMin),Pmax=c(panel$ScaleMax,panel$ShapeMax))
            Vprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,ltrunc=panel$ltrunc,rtrunc=panel$rtrunc,rev=panel$Irev)
            Mprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=MLEout$shape_hat,scale=MLEout$scale_hat,ltrunc=panel$ltrunc,rtrunc=panel$rtrunc,rev=panel$Irev)
            ksout<-ks.test(panel$data[panel$data>=panel$ltrunc & panel$data<=panel$rtrunc],"pWeibull",scale=MLEout$scale_hat,shape=MLEout$shape_hat,location=panel$location,ltrunc=panel$ltrunc,rtrunc=panel$rtrunc,rev=panel$Irev)
         }else{
            MLEout<-Weibull.MLE(data=panel$data,Params=c(panel$scale,panel$shape),location=panel$location,ltrunc=panel$ltrunc,rev=panel$Irev,Pmin=c(panel$ScaleMin,panel$ShapeMin),Pmax=c(panel$ScaleMax,panel$ShapeMax))
            Vprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,ltrunc=panel$ltrunc,rev=panel$Irev)
            Mprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=MLEout$shape_hat,scale=MLEout$scale_hat,ltrunc=panel$ltrunc,rev=panel$Irev)
            ksout<-ks.test(panel$data[panel$data>=panel$ltrunc],"pWeibull",scale=MLEout$scale_hat,shape=MLEout$shape_hat,location=panel$location,ltrunc=panel$ltrunc,rev=panel$Irev)
         }
      }else{
         if(panel$Irtrunc){
            MLEout<-Weibull.MLE(data=panel$data,Params=c(panel$scale,panel$shape),location=panel$location,rtrunc=panel$rtrunc,rev=panel$Irev,Pmin=c(panel$ScaleMin,panel$ShapeMin),Pmax=c(panel$ScaleMax,panel$ShapeMax))
            Vprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,rtrunc=panel$rtrunc,rev=panel$Irev)
            Mprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=MLEout$shape_hat,scale=MLEout$scale_hat,rtrunc=panel$rtrunc,rev=panel$Irev)
            ksout<-ks.test(panel$data[panel$data<=panel$rtrunc],"pWeibull",scale=MLEout$scale_hat,shape=MLEout$wshape_hat,location=panel$location,rtrunc=panel$rtrunc,rev=panel$Irev)
         }else{
            MLEout<-Weibull.MLE(data=panel$data,Params=c(panel$scale,panel$shape),location=panel$location,rev=panel$Irev,Pmin=c(panel$ScaleMin,panel$ShapeMin),Pmax=c(panel$ScaleMax,panel$ShapeMax))
            Vprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,rev=panel$Irev)
            Mprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=MLEout$shape_hat,scale=MLEout$scale_hat,rev=panel$Irev)
            ksout<-ks.test(panel$data,"pWeibull",scale=MLEout$scale_hat,shape=MLEout$shape_hat,location=panel$location,rev=panel$Irev)
         }
      }
      par(mfg=c(1,1))
      draw.fig1(panel,Vprobs)
      nn<-length(panel$Hbreaks)
      mm<-length(panel$data)
      Mprobs<-Mprobs[2:nn]-Mprobs[1:(nn-1)]
      xxx<-(panel$Hbreaks[2:nn]+panel$Hbreaks[1:(nn-1)])/2
      lines(xxx,mm*Mprobs,col="blue",lwd=2)
      par(mfg=c(2,1))
      draw.fig3(panel,Vprobs)
      Hcnnt<-hist(panel$data,panel$Hbreaks,plot=F)$counts
      xxx<-(panel$Hbreaks[2:nn]+panel$Hbreaks[1:(nn-1)])/2
      nn<-length(panel$data)
      points(xxx,Hcnnt-nn*Mprobs,pch=1,col="blue")
      par(mfg=c(2,2))
      draw.fig4(panel,Vprobs)
      polygon(c(-2,12,12,-2,-2),c(-1,-1,5,5,-1),col="white",border=NA)
      text(5,4,"MLE Estimates",cex=1.25)
      text(5,2,paste("Scale =",round(MLEout$scale_hat,4),"Shape =",round(MLEout$shape_hat,4)))
      text(5,0,paste("KS Test: D =",round(ksout$stat,4),"Pr(D) =",round(ksout$p.val,4)),cex=.95)
      return(panel)
   }
   get.Moments<-function(panel)
   {
      amean<-mean(panel$data)
      qmean<-sqrt(mean(panel$data^2))
      if(panel$Iltrunc){
         if(panel$Irtrunc){
            MOMout<-Weibull.Mom(qmean=qmean,amean=amean,scale=panel$scale,shape=panel$shape,location=panel$location,ltrunc=panel$ltrunc,rtrunc=panel$rtrunc,rev=panel$Irev)
            Vprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,ltrunc=panel$ltrunc,rtrunc=panel$rtrunc,rev=panel$Irev)
            Mprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=MOMout$shape_hat,scale=MOMout$scale_hat,ltrunc=panel$ltrunc,rtrunc=panel$rtrunc,rev=panel$Irev)
            ksout<-ks.test(panel$data,"pWeibull",scale=MOMout$scale_hat,shape=MOMout$shape_hat,location=panel$location,ltrunc=panel$ltrunc,rtrunc=panel$rtrunc,rev=panel$Irev)
         }else{
            MOMout<-Weibull.Mom(qmean=qmean,amean=amean,scale=panel$scale,shape=panel$shape,location=panel$location,ltrunc=panel$ltrunc,rev=panel$Irev)
            Vprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,ltrunc=panel$ltrunc,rev=panel$Irev)
            Mprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=MOMout$shape_hat,scale=MOMout$scale_hat[1],ltrunc=panel$ltrunc,rev=panel$Irev)
            ksout<-ks.test(panel$data,"pWeibull",scale=MOMout$scale_hat,shape=MOMout$shape_hat,location=panel$location,ltrunc=panel$ltrunc,rev=panel$Irev)
         }
      }else{
         if(panel$Irtrunc){
            MOMout<-Weibull.Mom(qmean=qmean,amean=amean,scale=panel$scale,shape=panel$shape,location=panel$location,rtrunc=panel$rtrunc,rev=panel$Irev)
            Vprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,rtrunc=panel$rtrunc,rev=panel$Irev)
            Mprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=MOMout$shape_hat,scale=MOMout$scale_hat,rtrunc=panel$rtrunc,rev=panel$Irev)
            ksout<-ks.test(panel$data,"pWeibull",scale=MOMout$scale_hat,shape=MOMout$shape_hat,location=panel$location,rtrunc=panel$rtrunc,rev=panel$Irev)
         }else{
            MOMout<-Weibull.Mom(qmean=qmean,amean=amean,scale=panel$scale,shape=panel$shape,location=panel$location,rev=panel$Irev)
            Vprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=panel$shape,scale=panel$scale,rev=panel$Irev)
            Mprobs<-pWeibull(panel$Hbreaks,location=panel$location,shape=MOMout$shape_hat,scale=MOMout$scale_hat,rev=panel$Irev)
            ksout<-ks.test(panel$data,"pWeibull",scale=MOMout$scale_hat,shape=MOMout$shape_hat,location=panel$location,rev=panel$Irev)
         }
      }
      par(mfg=c(1,1))
      draw.fig1(panel,Vprobs)
      nn<-length(panel$Hbreaks)
      mm<-length(panel$data)
      Mprobs<-Mprobs[2:nn]-Mprobs[1:(nn-1)]
      xxx<-(panel$Hbreaks[2:nn]+panel$Hbreaks[1:(nn-1)])/2
      lines(xxx,mm*Mprobs,col="blue",lwd=2)
      par(mfg=c(2,1))
      draw.fig3(panel,Vprobs)
      Hcnnt<-hist(panel$data,panel$Hbreaks,plot=F)$counts
      xxx<-(panel$Hbreaks[2:nn]+panel$Hbreaks[1:(nn-1)])/2
      nn<-length(panel$data)
      points(xxx,Hcnnt-nn*Mprobs,pch=1,col="blue")
      par(mfg=c(2,2))
      draw.fig4(panel,Vprobs)
      polygon(c(-2,12,12,-2,-2),c(-1,-1,5,5,-1),col="white",border=NA)
      text(5,4,"Moment Estimates",cex=1.25)
      text(5,2,paste("Scale =",round(MOMout$scale_hat,4),"Shape =",round(MOMout$shape_hat,4)))
      text(5,0,paste("KS Test: D =",round(ksout$stat,4),"Pr(D) =",round(ksout$p.val,4)),cex=.95)
      return(panel)
   }
   rp.messagebox("Written by Dr. John A. Kershaw, Jr., Faculty of Forestry and Env. Mgmt, University of New Brunswick")
   if(!missing(wt)){
      tdata<-data
      if(length(wt)!=length(data)){stop("length of weights must match length of data")}
      data<-NULL
      for(i in 1:length(wt)){
         data<-c(data,rep(tdata[i],round(wt[i],0)))
      }
      data<-data+runif(length(data),-.05,.05)
   }
   Xmin<-0
   Xmax<-10*round(max(data)/10 + .5,0)
   Hbreaks<-seq(0,Xmax,10)
   scale<-sqrt(mean(data^2))
   ShapeMin<-.1
   ShapeMax<-100
   ScaleMin<-0
   ScaleMax<-1000
   Wpanel<-rp.control(title="Weibull Parameter Finder",size=c(500,700),data=data,Hbreaks=Hbreaks,cwid=10,location=0,scale=scale,shape=3.6,ltrunc=0,rtrunc=Xmax,Iltrunc=FALSE,ShapeMin=ShapeMin,ShapeMax=ShapeMax,ScaleMin=ScaleMin,ScaleMax=ScaleMax,Irtrunc=FALSE,Irev=FALSE)
   rp.slider(Wpanel,cwid,1,100,hist.draw,"Class Width")
   rp.checkbox(Wpanel,Irev,action=do.reverse,labels="Reverse Weibull")
   rp.slider(Wpanel,location,0,Xmax,hist.draw,"Location Parameter")
   rp.slider(Wpanel,scale,ScaleMin,ScaleMax,hist.draw,"Scale Parameter")
   rp.slider(Wpanel,shape,ShapeMin,ShapeMax,hist.draw,"Shape Parameter")
   rp.checkbox(Wpanel,Iltrunc,action=hist.draw,labels="Left Truncation")
   rp.slider(Wpanel,ltrunc,0,Xmax,hist.draw,"Left Truncation Parameter")
   rp.checkbox(Wpanel,Irtrunc,action=hist.draw,labels="Right Truncation")
   rp.slider(Wpanel,rtrunc,0,Xmax,hist.draw,"Right Truncation Parameter")
   rp.doublebutton(Wpanel,ShapeMin,step=.1,title="Minimum value for Shape Parameter",action=hist.draw,initval=.1,range=c(.1,Inf))
   rp.doublebutton(Wpanel,ShapeMax,step=.1,title="Maximum value for Shape Parameter",action=hist.draw,initval=.1,range=c(1,Inf))
   rp.doublebutton(Wpanel,ScaleMin,step=1,title="Minimum value for Scale Parameter",action=hist.draw,initval=0,range=c(0,Inf))
   rp.doublebutton(Wpanel,ScaleMax,step=1,title="Maximum value for Scale Parameter",action=hist.draw,initval=1000,range=c(0,Inf))
   rp.button(Wpanel,get.MLEs,title="Get MLE Estimates")
   rp.button(Wpanel,get.Moments,title="Get Moment Estimates")
}
#
#  This file is the generalized Weibull MLE fitter
#
Weibull.MLE<-function(data,wt,Params,location,ltrunc,rtrunc,rev=F,Pmin,Pmax,minZero=T)
{
#
#   MLE fitter assumes Location Parameter, if required, is known
#
   LLF<-function(x,data,wt,location,ltrunc,rtrunc,rev)
   {
      if(missing(ltrunc)){
        if(missing(rtrunc)){
           llf<-sum(wt*log(dWeibull(data,scale=x[1],shape=x[2],location=location,rev=rev)))
        }else{
           llf<-sum(wt*log(dWeibull(data,scale=x[1],shape=x[2],rtrunc=rtrunc,location=location,rev=rev)))
        }
      }else{
        if(missing(rtrunc)){
           llf<-sum(wt*log(dWeibull(data,scale=x[1],shape=x[2],ltrunc=ltrunc,location=location,rev=rev)))
        }else{
           llf<-sum(wt*log(dWeibull(data,scale=x[1],shape=x[2],ltrunc=ltrunc,rtrunc=rtrunc,location=location,rev=rev)))
        }
      }
      return(-llf)
   }
#
#  setting bounds on parameter estimates
#
   if(missing(Pmin)){
      Pmin<-c(0,.1)
   }
   if(missing(Pmax)){
      Pmax<-c(Inf,Inf)
   }
   if(missing(location)){
      if(rev){location<-max(round(data+.5,0))}else{location<-0}
   }
   if(missing(wt)){wt<-rep(1,length(data))}
   if(missing(Params)){
      if(!rev){
         p1<-wt.quantile(data,wt,.63)
         p2<-wt.mean(data,wt)/gamma(1+(1/p1))
      }else{
         p1<-location-wt.quantile(data,wt,.63)
         p2<-(location-wt.mean(data,wt))/gamma(1+(1/p1))
      }
      p2<-3.6
      Params<-c(p1,p2)
      print(paste("Parameters set to initial values",p1,p2))
   }
   if(missing(ltrunc)){
     if(missing(rtrunc)){
        if(!rev){
           ltrunc<-0
           rtrunc<-Inf
        }else{
           if(minZero){ltrunc<-0}else{ltrunc<-(-Inf)}
           rtrunc<-location
        }
     }else{
        if(!rev){
           ltrunc<-0
        }else{
           if(minZero){ltrunc<-0}else{ltrunc<-(-Inf)}
        }
     }
   }else{
     if(missing(rtrunc)){
        if(!rev){
           rtrunc<-Inf
        }else{
           rtrunc<-location
        }
     }
   }
   temp<-nlminb(start=Params,objective=LLF,data=data,wt=wt,location=location,ltrunc=ltrunc,rtrunc=rtrunc,rev=rev,lower=Pmin,upper=Pmax)
   output<-data.frame(scale_hat=temp$par[1],shape_hat=temp$par[2])
   return(output)  
}
wt.quantile<-function(x,wt,quantile)
{
#
#  returns x corresponding to the bin containing the quantile of interest
#
   Xord<-order(x)
   x<-x[Xord]
   wt<-wt[Xord]
   Px<-cumsum(wt)/sum(wt)
   qX<-max(x[Px<=quantile])
   return(qX)
}
wt.mean<-function(x,wt){sum(x*wt)/sum(wt)}
Weibull.Mom<-function(qmean,amean,cvar,scale,shape,location,ltrunc,rtrunc,rev=F)
{
   if(missing(qmean)){
      if(missing(amean)){
         stop("quadratic mean or arithmetic mean must be specified")
      }else{
         if(missing(cvar)){
            stop("coefficient of variation must be specified when only arithmetic mean is specified")
         }else{
            qmean<-amean*sqrt(1+((cvar^2)/10000))
         }
      }
   }else{
      if(missing(amean)){
         if(missing(cvar)){
            stop("coefficient of variation must be specified when only quadratic mean is specified")
         }else{
            amean<-qmean/sqrt(1+((cvar^2)/10000))
         }
      
      }
   }
   print(paste(amean,qmean))
   require("BB")
   dtrunc <- function(x, spec, a = -Inf, b = Inf, ...)
   {
      tt <- rep(0, length(x))
      g <- get(paste("d", spec, sep = ""), mode = "function")
      G <- get(paste("p", spec, sep = ""), mode = "function")
      tt[x>=a & x<=b] <- g(x[x>=a&x<=b], ...)/(G(b, ...) - G(a, ...))
      return(tt)
   }
   extrunc <- function(spec, a = -Inf, b = Inf,...)
   {
      f <- function(x) x * dtrunc(x, spec, a = a, b = b, ...)
      return(integrate(f, lower = a, upper = b)$value)
   }
   vartrunc <- function(spec, a = -Inf, b = Inf, ...)
   {
      ex <- extrunc(spec, a = a, b = b, ...)
      f <- function(x) (x - ex)^2 * dtrunc(x, spec, a = a, b = b, ...)
      tt <- integrate(f, lower = a, upper = b)$value
      return(tt)
   }
   Moments<-function(p,qmean,amean,location,ltrunc,rtrunc,rev)
   {
      r<-rep(NA,2)
      r[1]<-amean-extrunc("Weibull",a=0,b=Inf,scale=p[1],shape=p[2],location=location,ltrunc=ltrunc,rtrunc=rtrunc,rev=rev)
      r[2]<-(qmean^2-amean^2)-vartrunc("Weibull",a=0,b=Inf,scale=p[1],shape=p[2],location=location,ltrunc=ltrunc,rtrunc=rtrunc,rev=rev)
      return(r)
   }
   if(!rev){
      if(missing(location)){location<-0}
      if(missing(ltrunc)){ltrunc<-location}
      if(missing(rtrunc)){rtrunc<-Inf}
   }else{
      if(missing(location)){stop("Location parameter must be specified for Reverse Weibull")}
      if(missing(ltrunc)){ltrunc<-0}
      if(missing(rtrunc)){rtrunc<-location}
   }
   if(missing(scale)){scale<-qmean}
   if(missing(shape)){shape<-3.6}
   par0<-c(scale,shape)
   temp<-BBsolve(par=par0,fn=Moments,qmean=qmean,amean=amean,location=location,ltrunc=ltrunc,rtrunc=rtrunc,rev=rev,quiet=T)
   output<-data.frame(scale_hat=temp$par[1],shape_hat=temp$par[2])
   return(output)
}
